#!/usr/bin/python
# coding: utf-8

__author__ = 'qasar'

"""
Toto je program, ktery stahne data o verejnych zakazkach z Vestniku verejnych zakazek
http://www.vestnikverejnychzakazek.cz/.

Stahuje pouze platné formuláře.
"""

##########################################################
#
# TODO
#
# * pokud načítání stránky-formuláře spadne, načíst ji znovu (nutno nastavit maximální počet pokusů,
#   než se to vzdá a spadení se zapíše do errfile.txt) - důvod je ten, že co jednou spadne, se příště
#   normálně načte
#
##########################################################


##########################################################
#
# import modulu
#
##########################################################

import time
import cookielib
import urllib2
import re
import lxml.html
import io
import unicodedata
import sys
import getopt
from pprobberdef import *



##########################################################
#
# zakladni konstanty
#
##########################################################

# den a hodina stazeni
curdate = time.strftime('%y%m%d_%H_%M')

# odkud kam se data stahují
datasince = '1.12.2014'
datato = '1.12.2014'
dataform = '3'

# jestli je standardní běh, kdy se tvoří minced meat (True), nebo se řeší jen soubory z errfile (False)
standard_run = True

# fyzická jména souborů
in_key_dict_name = 'pprobber_dictionary.txt'   # soubor, ze kterého se na začátku načtou už známá jména položek
out_key_dict_name = 'pprobber_dictionary.txt'  # soubor, do kterého se na konci zapíšou všechna známa jména položek
minced_meat_name = 'mincedmeat.txt'  # soubor, do kterého se uloží minced meat s daty
err_file_name = 'errfile.txt'  # soubor, kam se uloží url formulářů, které při stahování chcíply
input_err_file_name = 'inputerrfile.txt'  # soubor, ze kterého se v čtou odkazy formulářů, které chcíply (v recovery)

# základ webové stránky
web_base = 'http://www.vestnikverejnychzakazek.cz'

# počet chyb při načítání formulářů
number_of_errors = 0



##########################################################
#
# globalni proměnné
#
##########################################################

inner_page = -1  # pořadí zpracovávaného formuláře (bude 1, 2, ...)
outer_page = -1  # pořadí zpracovávané stránky se seznamem formulářů (bude 1, 2, ...)
num_inner_pages = -1  # počet celkově zpracovávaných formulářů -- zjistí se při prvním načtení seznamu formulářů



##########################################################
#
# slovnik
#
##########################################################

keydict = dict()


def add_key_to_dict(key, fieldtype, pageno):
    """
    Zapíše položku do slovníku známý položek (fields).

    :param key: string, jméno položky
    :param fieldtype: string, typ položky
    :param pageno: int, číslo formuláře, kde se key poprvé objevilo
    """
    global keydict
    key = re.sub('\[[0-9]+\]', '[*]', key)
    if not key in keydict:
        keydict[key] = (fieldtype, pageno)


def save_key_dict():
    """
    Uloží slovník položek (keys) do souboru.

    Uloží jak ty položky, které byly načteny na začáku programu, tak nově extrahované položky.
    """
    print('Writing key dictionary to ' + out_key_dict_name + '.')
    f = open(out_key_dict_name, 'w')
    for key, values in keydict.iteritems():
        name, pageno = values
        f.write(key + fieldsep + name + fieldsep + str(pageno) + '\n')
    f.close()


def load_key_dict():
    """
    Načte slovník položek ze souboru.
    """
    global keydict
    try:
        print('Reading key dictionary from ' + in_key_dict_name + '.')
        f = open(in_key_dict_name, 'r')
        for line in f:
            line = line.strip()
            line = line.split(fieldsep)
            key, name, pageno = line
            add_key_to_dict(key, name, int(pageno))
        f.close()
    except:
        print('... reading key dictionary failed!')


##########################################################
#
# kanal pro cteni stranek
#
##########################################################

# opener pro otevirani stranek
cj = cookielib.LWPCookieJar()
opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
urllib2.install_opener(opener)



##########################################################
#
# adresy webovych stranek, obrazku a ukladacich adresaru
#
##########################################################

# url vnejsich stranek -- ty obsahuji seznam jednotlivych zakazek
# http://www.vestnikverejnychzakazek.cz/cs/Searching/FullTextSearch?selectedFormType=3&selectedTypeForm=0&selectedLimit=0&selectedContractType=0&selectedSubjectOfActivity=all&selectedProcedureType=0&selectedContractingAuthorityAccordingToLaw=0&selectedActual=0&dateTimePublicationFrom=01%2F01%2F2014%2000%3A00%3A00&dateTimePublicationTo=11%2F15%2F2014%2000%3A00%3A00&selectedIsCurrent=0&size=20&page=644
def webpage_select(since, to, form, page):
    """
    Vrátí url stahované webové stránky.

    :param since: str, řetězec ve tvaru: 'd.m.rrrr' - datum, odkud se začíná stahovat
    :param to: str, řetězec ve tvaru: 'd.m.rrrr' - datum, až kam se bude stahovat
    :param form: str, ale musí jít převést na int, číslo formuláře, který se stahuje (3 nebo 6)
    :param page: str, ale musí jít převést na int, číslo stránky, která se bude stahovat
    :return: str, url stránky
    """
    sday, smonth, syear = since.split('.')
    tday, tmonth, tyear = to.split('.')
    syear = int(syear)
    smonth = int(smonth)
    sday = int(sday)
    tyear = int(tyear)
    tmonth = int(tmonth)
    tday = int(tday)
    page = int(page)
    form = int(form)
    # stahují se jen platné formuláře -- selectedIsCurrent=1 (všechny formuláře by měly 0 místo 1)
    page = ('http://www.vestnikverejnychzakazek.cz/cs/Searching/FullTextSearch?selectedFormType={7}&' +
            'selectedTypeForm=0&selectedLimit=0&selectedContractType=0&selectedSubjectOfActivity=all&' +
            'selectedProcedureType=0&selectedContractingAuthorityAccordingToLaw=0&selectedActual=0&' +
            'dateTimePublicationFrom={0:0>2d}%2F{1:0>2d}%2F{2}%2000%3A00%3A00&' +
            'dateTimePublicationTo={3:0>2d}%2F{4:0>2d}%2F{5}%2000%3A00%3A00&' +
            'selectedIsCurrent=1&size=20&page={6}').format(smonth, sday, syear,
                                                           tmonth, tday, tyear, page, form)
    return page


def url_request(url):
    """
    Vrací request pro url.

    :param url: str
    """
    thedata = None
    theheaders = {'User-agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
    req = urllib2.Request(url, thedata, theheaders)  # create a request object
    return req


def webpage_request(since, to, form, page):
    """
    Vrací request na webovou stránku se seznamem formulářů.

    :param since:
    :param to:
    :param form:
    :param page:
    :return:
    """
    theurl = webpage_select(since, to, form, page)
    # thedata = None
    # theheaders = {'User-agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
    # req = urllib2.Request(theurl, thedata, theheaders)  # create a request object
    req = url_request(theurl)
    return req


def strip_accents(s):
    """
    Odstraní z řetězce akcenty.

    :param s: str
    :return: str
    """
    if isinstance(s, basestring):
        if not isinstance(s, unicode):
            s = unicode(s, "utf-8")
    s = ''.join(c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn')
    s = s.encode("utf-8")
    return s



##########################################################
#
# funkce pro stazeni a zpracovani jedne webove stranky
#
##########################################################

def process_inner_page(pageurl):
    """
    Zpracuje jeden formulář a zapíše jej do minced meatu.

    :param pageurl: str, url zpracovávaného formuláře
    """
    export_string = ''  # řetězec, kam se ukládá minced meat před tím, než se zapíše do souboru
    pageno = pageurl.split("/")[-1]
    # thedata = None
    # theheaders = {'User-agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
    # req = urllib2.Request(pageurl, thedata, theheaders)  # create a request object
    req = url_request(pageurl)
    page_file = urllib2.urlopen(req)  # and open it to return a handle on the url
    the_page = page_file.read()
    page_file.close()
    parsed_file = lxml.html.parse(io.BytesIO(the_page))
    doc = parsed_file.getroot()
    #
    # ulož url stránky
    export_string += 'ROBBER_URL' + fieldsep + 'robberurl' + fieldsep + pageurl + '\n'
    add_key_to_dict('ROBBER_URL', 'robberurl', pageno)
    #
    # procesuj hlavičku
    header = doc.findall(".//div[@class='iform-header']")[0]
    header = lxml.html.tostring(header, encoding="utf-8")
    #
    header = re.sub('</*div.*?>', '', header)
    header = re.sub('</*span.*?>', '', header)
    header = re.sub('\s+', ' ', header)
    fields = header.split('<br>')
    for field in fields:
        field = field.rstrip()
        if len(field) > 0:
            ss = field.split(':')
            name = ss[0].strip()
            if len(ss) > 1:
                value = ss[1].strip()
            else:
                value = ss[0].strip()
            header_name = 'ROBBER_HEADER_' + strip_accents(name.replace(' ', '_'))
            export_string += header_name + fieldsep + "robberheader" + fieldsep + value + fieldsep + name + '\n'
            add_key_to_dict(header_name, 'robberheader', pageno)
    #
    # procesuj <input...>
    fieldsets = doc.findall(".//fieldset")
    for fieldset in fieldsets:
        inputfields = fieldset.findall(".//input[@class='input-readonly']")
        for inputfield in inputfields:
            fieldname = inputfield.get('name')
            fieldtype = inputfield.get('type')
            fieldvalue = inputfield.get('value')
            fieldvalue = re.sub('\n', ' ', fieldvalue)
            fieldexport = '== unkown type =='
            if fieldtype == 'text':
                fieldexport = fieldname + fieldsep + fieldtype + fieldsep + fieldvalue
            if fieldtype == "checkbox":
                if inputfield.get('checked') == 'checked':
                    fieldchecked = "True"
                else:
                    fieldchecked = "False"
                fieldexport = fieldname + fieldsep + fieldtype + fieldsep + fieldchecked
            if fieldtype == 'radio':
                if inputfield.get('checked') is None:
                    fieldchecked = "False"
                else:
                    fieldchecked = "True"
                fieldexport = fieldname + fieldsep + fieldtype + fieldsep + fieldvalue + fieldsep + fieldchecked
            add_key_to_dict(fieldname, fieldtype, pageno)
            export_string += fieldexport.encode('utf8') + '\n'
        #
        # procesuj <select>
        inputfields = fieldset.findall(".//select")
        for inputfield in inputfields:
            selectname = inputfield.get('name')
            selecttype = "select"
            select = inputfield.findall(".//option[@selected='selected']")
            if len(select) == 0:
                selectvalue = "<NA>"
            else:
                selectvalue = select[0].get("value")
            fieldexport = selectname + fieldsep + selecttype + fieldsep + selectvalue
            add_key_to_dict(selectname, selecttype, pageno)
            export_string += fieldexport.encode('utf8') + '\n'
    mincedmeat.write(export_string + recordsep + '\n')


def process_outer_page(since, to, form, page):
    """
    Stáhne a zpracuje jednu webovou stránku se seznamem formulářů.

    Volá process_inner_page(), aby zpracovalo jednotlivé vlastní formuláře.

    :param since: str, řetězec ve tvaru: 'd.m.rrrr' - datum, odkud se začíná stahovat
    :param to: str, řetězec ve tvaru: 'd.m.rrrr' - datum, až kam se bude stahovat
    :param form: str, ale musí jít převést na int, číslo formuláře, který se stahuje (3 nebo 6)
    :param page: str, ale musí jít převést na int, číslo stránky, která se bude stahovat
    :return:
    """
    global inner_page, outer_page, web_base, number_of_errors
    req = webpage_request(since, to, form, page)
    page_file = urllib2.urlopen(req)
    the_page = page_file.read()
    page_file.close()
    parsed_file = lxml.html.parse(io.BytesIO(the_page))
    doc = parsed_file.getroot()
    table = doc.findall(".//table")
    rows = table[0].findall(".//tr")
    for row in rows[1:]:
        columns = row.findall(".//td")
        link = columns[1].findall(".//a")
        processed_url = web_base + link[0].get('href')
        print(processed_url)
        try:
            process_inner_page(processed_url)
        except:
            print('... processing the previous page failed!')
            errfile.write(processed_url + '\n')
            number_of_errors += 1
        inner_page += 1
    outer_page += 1
    return the_page


def process_forms(since, to, form):
    """
    Prochází výběr formulářů tak, že začíná od data since po datum to a stahuje formuláře form.

    :param since: str, řetězec ve tvaru: 'd.m.rrrr' - datum, odkud se začíná stahovat
    :param to: str, řetězec ve tvaru: 'd.m.rrrr' - datum, až kam se bude stahovat
    :param form: str, ale musí jít převést na int, číslo formuláře, který se stahuje (3 nebo 6)
    """
    global inner_page, outer_page, num_inner_pages
    inner_page = 1
    outer_page = 1
    num_inner_pages = -1
    while inner_page <= num_inner_pages or num_inner_pages < 0:
        if num_inner_pages > 0:
            print('--- Processing form ' + str(inner_page) + ' out of ' + str(num_inner_pages) + ' forms. ---')
        else:
            print('--- Processing first twenty forms. ---')
        the_page = process_outer_page(since, to, form, outer_page)
        num_inner_pages = re.findall(
            '<div class="t-status-text">Zobrazuj&#237; se položky (\d+?) až (\d+?) z (\d+?)</div>', the_page)
        num_inner_pages = int(num_inner_pages[0][2])
        # num_inner_pages = 51  # LADICI -- ODSTRANIT



##########################################################
#
# vlastni telo programu
#
##########################################################

def process_command_line(argv):
    """
    Zpracuje parametry z příkazového řádku
    """
    global datasince, datato, dataform, in_key_dict_name, out_key_dict_name, minced_meat_name, err_file_name,\
        input_err_file_name, standard_run
    try:
        opts, args = getopt.getopt(argv, 'hs:t:f:i:o:m:e:r:',
                                   ['since=', 'to=', 'form=', 'inputkeydict=', 'outputkeydict=', 'mincedmeat=',
                                    'errorfile=', 'inputerrfile=', 'process_failed_forms'])
    except getopt.GetoptError:
        print('pprob.py -s <since> -t <to> -f <form> -i <inputkeydict> -o <outputkeydict> ' +
              '-m <mincedmeat> -e <errorfile> -r <inputerrfile> --process_failed_forms')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('pprob.py -s <since> -t <to> -f <form> -i <inputkeydict> -o <outputkeydict> ' +
                  '-m <mincedmeat> -e <errorfile> -r <inputerrfile> --process_failed_forms')
            sys.exit()
        elif opt in ('-s', '--since'):
            datasince = arg
        elif opt in ('-t', '--to'):
            datato = arg
        elif opt in ('-f', '--form'):
            dataform = arg
        elif opt in ('-i', '--inputkeydict'):
            in_key_dict_name = arg
        elif opt in ('-o', '--outputkeydict'):
            out_key_dict_name = arg
        elif opt in ('-m', '--mincedmeat'):
            minced_meat_name = arg
        elif opt in ('-e', '--errorfile'):
            err_file_name = arg
        elif opt in ('-r', '--inputerrfile'):
            input_err_file_name = arg
        elif opt == '--process_failed_forms':
            standard_run = False


def process_failed_forms():
    """
    Tato funkce se spouští v režimu, kdy se čte starý errfile a stahují se stránky, které předtím selhaly.
    """
    global number_of_errors
    f = open(input_err_file_name, 'r')
    for processed_url in f:
        processed_url = processed_url.strip()
        print(processed_url)
        try:
            process_inner_page(processed_url)
        except:
            print('... processing the previous page failed!')
            errfile.write(processed_url + '\n')
            number_of_errors += 1
    f.close()


def main(argv):
    """
    Vlastní tělo programu.
    """
    process_command_line(argv)
    global mincedmeat, errfile
    print('I will try to read form ' + str(dataform) + ' since ' + datasince + ' to ' + datato + '.')
    print('Minced meat data will be written to ' + minced_meat_name + '.')
    mincedmeat = open(minced_meat_name, 'w')
    errfile = open(err_file_name, 'w')
    load_key_dict()
    # buď se spustí standardní běh, nebo se dotáhnou stránky, které předtím selhaly
    if standard_run:
        process_forms(datasince, datato, int(dataform))
    else:
        process_failed_forms()
    save_key_dict()
    errfile.close()
    mincedmeat.close()
    print('Done.')
    print('Data written to minced meat file ' + minced_meat_name + '.')
    print('Number of errors: ' + str(number_of_errors) + '.')
    if number_of_errors > 0:
        print('Error messages written to ' + err_file_name + '.')


if __name__ == '__main__':
    main(sys.argv[1:])
