#!/usr/bin/python
# coding: utf-8

__author__ = 'qasar'

"""
Toto je program, ktery vezme mincedmeat dat o verejnych zakazkach z Vestniku verejnych zakazek
http://www.vestnikverejnychzakazek.cz/ a udela z nich csv soubor.
"""

##########################################################
#
# import modulu
#
##########################################################

import re
import sys
import getopt
from pprobberdef import *



##########################################################
#
# globalni proměnné
#
##########################################################

processed_list = []  # seznam jmen jednotlivých položek, jak jsou v minced meatu, tj. ve webových formulářích
type_list = []  # seznam typů jednotlivých položek
# name_list = []  # seznam jmen, která se vypíší do hlavičky csv souboru; nepoužívá se

# jména souborů
csv_def_name = 'csvdef.txt'  # jméno definičního souboru, co se mele do csv
input_file_name = 'mincedmeat.txt'  # jméno minced meat souboru s daty ke zpracování
output_file_name = 'csvfile.csv'  # jméno výsledného csv souboru


def read_csv_def_file(csv_def_name):
    """
    Tato funkce načte soubor, který definuje, jaká data a v jakém pořadí se zapíšou do výstupního csv souboru.

    :param csv_def_name: str, jméno souboru, ze kterého se čtou definice toho, co půjde do csv souboru
    """
    global processed_list, type_list, name_to_header  #, name_list
    f = open(csv_def_name, 'r')
    header_line = ''
    for line in f:
        line = line.strip()
        if len(line) > 0:
            line = line.split(fieldsep)
            field_form_name = line[0]
            field_type = line[1]
            if len(line) > 2:
                field_output_name = line[2]
            else:
                field_output_name = field_form_name
            processed_list.append(field_form_name)
            type_list.append(field_type)
            # name_list.append(field_output_name)
            name_to_header = re.sub('\[\*\]', csvfieldreplace, field_output_name)
            header_line = header_line + csvfieldsep + name_to_header
    header_line = header_line[len(csvfieldsep):]
    f.close()
    return header_line + '\n'


def process_minced_meat(input_file_name, csvfile):
    """
    Zpracuje minced meat a výsledek zapíše do csv souboru.

    :param input_file_name: str, jméno souboru s minced meat
    :param csvfile: file; csv soubor, kam se zapisují výsledná data
    """
    # global line, var, outline, item, field_form_name, form_type, form_value
    f = open(input_file_name, 'r')
    slovnik = dict()
    variants = set()
    minced_meat_line = 0
    for line in f:
        minced_meat_line += 1
        # print(minced_meat_line)  # pro ladící účely -- tiskne zpracované řádky -- umožní najít chybu v minced meatu
        line = line.strip()
        if line == recordsep:
            slovnik['ROBBER_VARIANTS_NUMBER'] = '"' + str(len(variants)) + '"'
            # pokud je konec záznamu, vypiš vybrané položky do csv souboru
            for var in variants:
                slovnik['ROBBER_VARIANT[' + str(var) + ']'] = '"' + str(var) + '"'
                outline = ''
                for item in processed_list:
                    item = re.sub('\[\*\]', '[' + str(var) + ']', item)
                    if item in slovnik:
                        outline = outline + csvfieldsep + slovnik[item]
                    else:
                        outline = outline + csvfieldsep + field_not_found_sign
                outline = outline[len(csvfieldsep):]
                # print(outline)
                csvfile.write(outline + '\n')
            slovnik = dict()
            variants = set()
        else:
            # pokud není konec souboru, zapamatuj si právě zpracovávanou položku minced meatu
            line += ' '
            line = line.split(fieldsep)
            field_form_name = line[0].strip()
            form_type = line[1].strip()
            var = re.findall('\[\d+\]', field_form_name)
            if len(var) > 0:
                var = int(var[0].strip('[').strip(']'))
                variants.add(var)
            if form_type in ('text', 'robberheader', 'select', 'checkbox'):
                # TODO přidat kontrolu typu
                form_value = line[2].strip()
                if len(form_value) == 0:
                    form_value = 'NA'
                else:
                    form_value = re.sub('"', "''", form_value)  # když tam jsou uvozovky, nahradí se kvůli čtení csv
                    form_value = '"' + form_value + '"'
                slovnik[field_form_name] = form_value
            # # TODO přidat kontrolu typu
            #     slovnik[form_name] = '"' + line[2] + '"'
            elif form_type == 'radio':
                if line[3].strip() == 'True':
                    slovnik[field_form_name] = '"' + line[2] + '"'
    f.close()


def process_command_line(argv):
    """
    Zpracuje parametry z příkazového řádku
    """
    global input_file_name, output_file_name, csv_def_name
    try:
        opts, args = getopt.getopt(argv, 'hi:o:d:',
                                   ['input=', 'output=', 'deffile='])
    except getopt.GetoptError:
        print('ppsteal.py -i <input_minced_meat> -o <output_cvf_file> -d <csv_def_file>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('ppsteal.py -i <input_minced_meat> -o <output_cvf_file> -d <csv_def_file>')
            sys.exit()
        elif opt in ('-i', '--input'):
            input_file_name = arg
        elif opt in ('-o', '--output'):
            output_file_name = arg
        elif opt in ('-d', '--deffile'):
            csv_def_name = arg


def main(argv):
    """
    Vlastní tělo programu.
    """
    process_command_line(argv)
    header = read_csv_def_file(csv_def_name)
    csvfile = open(output_file_name, 'w')
    csvfile.write(header)
    process_minced_meat(input_file_name, csvfile)
    csvfile.close()



if __name__ == '__main__':
    main(sys.argv[1:])
